module myterminal_top (
	// Clock and reset
	input wire i_CLK24,
	input wire i_USER_L,

	// Serial port
	input wire i_UART_RX,
	output wire o_UART_CTS,
	
	// LVDS output
	output wire o_LVDS_TX1,
	output wire o_LVDS_TX2,		
	output wire o_LVDS_TX3,	
	output wire o_LVDS_CLK	 
);

`include "constant.v"

wire [5:0] w_lcd_R, w_lcd_G, w_lcd_B;

wire reset_n = i_USER_L;
wire clk40;


wire w_hsync, w_vsync, w_den;


clock clock (
	.refclk (i_CLK24),
	.reset (~reset_n),			
	.clk1_out (clk40)
);


LVDS_7to1_TX lvds_out (
	.CLK_Tx (clk40),
	.RST_Tx (~reset_n),
              
	.T0_in ({w_lcd_G[0],w_lcd_R[5:0]}),
	.T1_in ({w_lcd_B[1:0],w_lcd_G[5:1]}),
	.T2_in ({w_den,w_vsync,w_hsync,w_lcd_B[5:2]}),
	.T3_in (7'b0000000),
              
	.TCLK_out (o_LVDS_CLK),
	.T0_out (o_LVDS_TX1),
	.T1_out (o_LVDS_TX2),
	.T2_out (o_LVDS_TX3),	
	.T3_out (open)
);



wire [7:0] in_byte;
wire in_byte_available;
serial_in #(
	.CLK_FREQUENCY_HZ (40_000_000),
	.SERIAL_BPS (115_200)
) serial_in (
	.clk (clk40),
	.reset (~reset_n),
	.rx (i_UART_RX),
	.data (in_byte),
	.oe (in_byte_available)
);

wire [14:0] font_address;
wire [15:0] char_row_bitmap;
font font (
	.clk (clk40),
	.font_address (font_address),
	.char_row_bitmap (char_row_bitmap)
);

wire [7:0] unicode;
wire unicode_available;
simple_fifo #(
	.DATA_WIDTH (8)
) simple_fifo (
	.clk (clk40),
	.reset (~reset_n),

	.in_data (in_byte),
	.in_data_available (in_byte_available),

	.receiver_ready (~o_UART_CTS),
	.out_data_available (unicode_available),
	.out_data (unicode)
);

wire [22:0] wr_address;
wire [31:0] wr_data;
wire [3:0] wr_mask;
wire wr_request;
wire wr_done;
wire [8:0] wr_burst_length;
wire [3:0] register_index;
wire [22:0] register_value;
terminal_stream terminal_stream (
	.clk (clk40),
	.reset (~reset_n),
	.ready_n (o_UART_CTS),
	.unicode (unicode),
	.unicode_available (unicode_available),
	.wr_address (wr_address),
	.wr_request (wr_request),
	.wr_data (wr_data),
	.wr_mask (wr_mask),
	.wr_done (wr_done),
	.wr_burst_length (wr_burst_length),
	.register_index (register_index),
	.register_value (register_value)
);

wire rd_request;
wire [22:0] rd_address;
wire [31:0] rd_data;
wire rd_available;
wire [8:0] rd_burst_length;
ram #(
	.CLK_FREQUENCY_HZ (40_000_000)
) ram (
	.clk (clk40),
	.rst_n (reset_n),

	// Read signals
	.rd_request (rd_request),
	.rd_address (rd_address),
	.rd_available (rd_available),
	.rd_data (rd_data),
	.rd_burst_length (rd_burst_length),

	// Write signals
	.wr_request (wr_request),
	.wr_mask (wr_mask),
	.wr_done (wr_done),
	.wr_address (wr_address),
	.wr_data (wr_data),
	.wr_burst_length (wr_burst_length)
);




video_controller video_controller (
	.clk (clk40),
	.reset (~reset_n),

	.video_hsync (w_hsync),
	.video_vsync (w_vsync),	
	.video_daten (w_den),
	.video_red (w_lcd_R),
	.video_green (w_lcd_G),
	.video_blue (w_lcd_B),			

	.rd_request (rd_request),
	.rd_address (rd_address),
	.rd_available (rd_available),
	.rd_data (rd_data),
	.rd_burst_length (rd_burst_length),

	.font_address (font_address),
	.char_row_bitmap (char_row_bitmap),

	.register_index (register_index),
	.register_value (register_value)
);

endmodule
